<?php

use Mockery as m;

/**
 * Class SquireBuilderTest
 *
 * Test the SquireFile and SquireImage classes
 */
class SquireBuilderTest extends SquireTestCase {

  /**
   * @var string The file key used for testing
   */
  private $key = 'test.txt';

  /**
   * Test that the SquireClient->file method returns an instance of SquireFile
   */
  public function testGetFileBuilder() {
    $builder = $this->getSquire()->file($this->key);
    $this->assertInstanceOf('Myriad\Squire\SquireFile', $builder);
  }

  /**
   * Test that the param method properly adds a parameter
   */
  public function testParamMethodAddsParam() {
    $url = $this->getFile()->param('foo', 'bar');
    $this->assertBuiltUrl($url, ['foo' => 'bar']);
  }

  /**
   * Test that a magic parameter properly adds a parameter
   */
  public function testMagicParamAddsParam() {
    $url = $this->getFile()->foo('bar');
    $this->assertBuiltUrl($url, ['foo' => 'bar']);
  }

  /**
   * Test that the url is built properly without any parameters.
   */
  public function testWithoutParameters() {
    $url = $this->getFile();
    $this->assertBuiltUrl($url);
  }

  /**
   * Test that a relative url is built properly without any parameters.
   */
  public function testRelativeWithoutParameters() {
    $url = $this->getFile()->build(true);
    $this->assertBuiltUrl($url, [], null, true);
  }

  /**
   * Test that a relative url is built properly with parameters.
   */
  public function testRelativeWithParameters() {
    $url = $this->getFile()->param('foo', 'bar')->build(true);
    $this->assertBuiltUrl($url, ['foo' => 'bar'], null, true);
  }

  /**
   * Test that the SquireClient->image method returns an instance of SquireImage
   */
  public function testGetImageBuilder() {
    $builder = $this->getSquire()->image($this->key);
    $this->assertInstanceOf('Myriad\Squire\SquireImage', $builder);
  }

  /**
   * Test that the height and width modifiers append the correct url parameters.
   */
  public function testWidthAndHeight() {
    $url = $this->getImage()->width(200)->height(300);
    $this->assertBuiltUrl($url, ['w' => '200', 'h' => '300']);
  }

  /**
   * Test that the fit modifiers append the correct url parameters.
   */
  public function testFit() {
    $url = $this->getImage()->fit('foo');
    $this->assertBuiltUrl($url, ['fit' => 'foo']);

    $url = $this->getImage()->fitContain();
    $this->assertBuiltUrl($url, ['fit' => 'contain']);

    $url = $this->getImage()->fitMax();
    $this->assertBuiltUrl($url, ['fit' => 'max']);

    $url = $this->getImage()->fitStretch();
    $this->assertBuiltUrl($url, ['fit' => 'stretch']);

    $url = $this->getImage()->fitCrop();
    $this->assertBuiltUrl($url, ['fit' => 'crop']);
  }

  /**
   * Test that the crop modifiers append the correct url parameters.
   */
  public function testCrop() {
    $url = $this->getImage()->crop('foo');
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'foo']);

    $url = $this->getImage()->cropTopLeft();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'top-left']);

    $url = $this->getImage()->cropTop();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'top']);

    $url = $this->getImage()->cropTopRight();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'top-right']);

    $url = $this->getImage()->cropLeft();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'left']);

    $url = $this->getImage()->cropCenter();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'center']);

    $url = $this->getImage()->cropRight();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'right']);

    $url = $this->getImage()->cropBottomLeft();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'bottom-left']);

    $url = $this->getImage()->cropBottom();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'bottom']);

    $url = $this->getImage()->cropBottomRight();
    $this->assertBuiltUrl($url, ['fit' => 'crop', 'crop' => 'bottom-right']);
  }

  /**
   * Test that the rect modifier append the correct url parameter.
   */
  public function testRect() {
    $url = $this->getImage()->rect(1, 2, 3, 4);
    $this->assertBuiltUrl($url, ['rect' => '1,2,3,4']);
  }

  /**
   * Test that the rotation modifiers append the correct url parameters.
   */
  public function testRotate() {
    $url = $this->getImage()->rotate('foo');
    $this->assertBuiltUrl($url, ['or' => 'foo']);

    $url = $this->getImage()->rotateAuto();
    $this->assertBuiltUrl($url, ['or' => 'auto']);

    $url = $this->getImage()->rotateLeft();
    $this->assertBuiltUrl($url, ['or' => '270']);

    $url = $this->getImage()->rotateRight();
    $this->assertBuiltUrl($url, ['or' => '90']);

    $url = $this->getImage()->rotateUpsideDown();
    $this->assertBuiltUrl($url, ['or' => '180']);
  }

  /**
   * Test that the brightness modifier append the correct url parameter.
   */
  public function testBrightness() {
    $url = $this->getImage()->brightness(50);
    $this->assertBuiltUrl($url, ['bri' => '50']);
  }

  /**
   * Test that the contrast modifier append the correct url parameter.
   */
  public function testContrast() {
    $url = $this->getImage()->contrast(50);
    $this->assertBuiltUrl($url, ['con' => '50']);
  }

  /**
   * Test that the gamma modifier append the correct url parameter.
   */
  public function testGamma() {
    $url = $this->getImage()->gamma(8.88);
    $this->assertBuiltUrl($url, ['gam' => '8.88']);
  }

  /**
   * Test that the sharpen modifier append the correct url parameter.
   */
  public function testSharpen() {
    $url = $this->getImage()->sharpen(50);
    $this->assertBuiltUrl($url, ['sharp' => '50']);
  }

  /**
   * Test that the blur modifier append the correct url parameter.
   */
  public function testBlur() {
    $url = $this->getImage()->blur(50);
    $this->assertBuiltUrl($url, ['blur' => '50']);
  }

  /**
   * Test that the pixelate modifier append the correct url parameter.
   */
  public function testPixel() {
    $url = $this->getImage()->pixel(50);
    $this->assertBuiltUrl($url, ['pixel' => '50']);
  }

  /**
   * Test that the filter modifiers append the correct url parameters.
   */
  public function testFilters() {
    $url = $this->getImage()->filter('foo');
    $this->assertBuiltUrl($url, ['filt' => 'foo']);

    $url = $this->getImage()->grayscale();
    $this->assertBuiltUrl($url, ['filt' => 'grayscale']);

    $url = $this->getImage()->sepia();
    $this->assertBuiltUrl($url, ['filt' => 'sepia']);
  }

  /**
   * Test that the quality modifiers append the correct url parameters.
   */
  public function testQuality() {
    $url = $this->getImage()->quality(20);
    $this->assertBuiltUrl($url, ['q' => '20', 'fm' => 'jpg']);
  }

  /**
   * Test that the format modifiers append the correct url parameters.
   */
  public function testFormat() {
    $url = $this->getImage()->format('foo');
    $this->assertBuiltUrl($url, ['fm' => 'foo']);

    $url = $this->getImage()->jpg();
    $this->assertBuiltUrl($url, ['fm' => 'jpg']);

    $url = $this->getImage()->png();
    $this->assertBuiltUrl($url, ['fm' => 'png']);

    $url = $this->getImage()->gif();
    $this->assertBuiltUrl($url, ['fm' => 'gif']);
  }

  /**
   * Returns a SquireImage instance for the given key.
   *
   * @param null|string $key
   *
   * @return \Myriad\Squire\SquireImage
   */
  private function getImage($key = null) {
    return $this->getSquire()->image($key ?: $this->key);
  }

  /**
   * Returns a SquireFile instance for the given key.
   *
   * @param null|string $key
   *
   * @return \Myriad\Squire\SquireFile
   */
  private function getFile($key = null) {
    return $this->getSquire()->file($key ?: $this->key);
  }

  /**
   * Asserts the build url has exactly the expected parameters.
   *
   * @param string|\Myriad\Squire\SquireFile $actualUrl
   * @param array $expectedParams
   * @param null|string $key
   * @param bool $relative
   */
  private function assertBuiltUrl($actualUrl, $expectedParams = [], $key = null, $relative = false) {
    $expectedUrl = trim($key ?: $this->key, '/');
    if (!empty($expectedParams)) {
      ksort($expectedParams);
      $expectedUrl .= '?' . http_build_query($expectedParams);
    }
    if (!$relative) {
      $expectedUrl = 'https://' . $this->confDomain . '/' . $this->confPrefix . '/' . $expectedUrl;
    }
    $this->assertEquals($expectedUrl, (string)$actualUrl);
  }

}