<?php

use Mockery as m;

class SquireClientTest extends SquireTestCase {

  private $key = 'test.txt';

  /**
   * Test the get method to ensure the proper resource is returned.
   */
  public function testGet() {
    $expectedResource = mt_rand();

    $request = $this->getMockGuzzleRequestInterface();
    $guzzle = $this->getMockGuzzleClient();
    $guzzle->shouldReceive('createRequest')->with('GET', $this->confUrl . '/api/v1/files/' . $this->key, [
      'headers' => [
        'Authorization' => $this->confToken
      ]
    ])->andReturn($request);

    $stream = m::mock('GuzzleHttp\Stream\GuzzleHttp\Stream');
    $stream->shouldReceive('detach')->andReturn($expectedResource);

    $response = m::mock('GuzzleHttp\Message\MessageInterface');
    $response->shouldReceive('getBody')->andReturn($stream);

    $guzzle->shouldReceive('send')->with($request)->andReturn($response);

    $resource = $this->getSquire(null, null, $guzzle)->get($this->key);
    $guzzle->shouldHaveReceived('send');

    $this->assertEquals($expectedResource, $resource);
  }

  /**
   * Test the put method with a resource.
   */
  public function testPutWithResource() {
    $resource = fopen('php://temp', 'r+');
    fwrite($resource, 'hello');

    $request = $this->getMockGuzzleRequestInterface();
    $guzzle = $this->getMockGuzzleClient();
    $guzzle->shouldReceive('createRequest')->with('PUT', $this->confUrl . '/api/v1/files/' . $this->key, [
      'body' => $resource,
      'headers' => [
        'Authorization' => $this->confToken
      ]
    ])->andReturn($request);
    $guzzle->shouldReceive('send')->with($request);
    $this->getSquire(null, null, $guzzle)->put($resource, $this->key);
    $guzzle->shouldHaveReceived('send');

    @fclose($resource);
  }

  /**
   * Test the delete method.
   */
  public function testDelete() {
    $request = $this->getMockGuzzleRequestInterface();

    $guzzle = $this->getMockGuzzleClient();
    $guzzle->shouldReceive('createRequest')->with('DELETE', $this->confUrl . '/api/v1/files/' . $this->key, [
      'headers' => [
        'Authorization' => $this->confToken
      ]
    ])->andReturn($request);
    $guzzle->shouldReceive('send')->with($request);

    $this->getSquire(null, null, $guzzle)->delete($this->key);
    $guzzle->shouldHaveReceived('send');
  }

  /**
   * Test generating a key without a prefix.
   */
  public function testGenerateKeyWithoutPrefix() {
    $key = $this->getSquire()->generateKey('txt');
    $this->assertGeneratedKey($key, 'txt');
  }

  /**
   * Test generating a key with a prefix.
   */
  public function testGenerateKeyWithPrefix() {
    $key = $this->getSquire()->generateKey('txt', 'my-');
    $this->assertGeneratedKey($key, 'txt', 'my-');
  }

  /**
   * Test generating a key from an SplFileInfo
   */
  public function testGenerateKeyFromFile() {
    $file = m::mock('\SplFileInfo');
    $file->shouldReceive('getExtension')->andReturn('png');
    $key = $this->getSquire()->generateKeyFromFile($file);
    $this->assertGeneratedKey($key, 'png');
  }

  /**
   * Test generating a key from an UploadedFile
   */
  public function testGenerateKeyFromUploadedFile() {
    $file = m::mock('\Symfony\Component\HttpFoundation\File\UploadedFile');
    $file->shouldReceive('getClientOriginalExtension')->andReturn('jpg');
    $key = $this->getSquire()->generateKeyFromFile($file);
    $this->assertGeneratedKey($key, 'jpg');
  }

  /**
   * Test generating a key from an UploadedFile that does not have the getClientOriginalExtension method.
   */
  public function testGenerateKeyFromOldUploadedFile() {
    $file = m::mock('\Symfony\Component\HttpFoundation\File\UploadedFile');
    $file->shouldIgnoreMissing()->shouldReceive('getClientOriginalName')->andReturn('test.gif');
    $key = $this->getSquire()->generateKeyFromFile($file);
    $this->assertGeneratedKey($key, 'gif');
  }

  /**
   * Assert a generated key includes the prefix, a v4 uuid, and the file extension.
   *
   * @param string $actualKey
   * @param string $extension
   * @param string $prefix
   */
  private function assertGeneratedKey($actualKey, $extension, $prefix = '') {
    if (!empty($prefix)) {
      $prefix = preg_quote($prefix);
    }
    $regex = '/' . $prefix . '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}\\.' . $extension . '/';
    $this->assertRegExp($regex, $actualKey, 'Generated key must be a uuid prefixed with the prefix and suffixed with the file extension.');
  }

}